window.Gleam.initSearch([{"doc":"glitr","title":"glitr","content":"# glitr\r\n\r\nGleam typed routes library for type-safe fullstack development\r\n\r\n[![Package Version](https://img.shields.io/hexpm/v/glitr)](https://hex.pm/packages/glitr)\r\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/glitr/)\r\n\r\n```sh\r\ngleam add glitr\r\n```\r\n```gleam\r\nimport glitr\r\nimport glitr/utils\r\nimport gleam/json\r\nimport gleam/dynamic\r\nimport gleam/http\r\n\r\npub type Todo {\r\n  Todo(id: String, title: String)\r\n}\r\n\r\nfn encoder(t: Todo) -&gt; json.Json {\r\n  json.object([\r\n    #(&quot;id&quot;, json.string(t.id)),\r\n    #(&quot;title&quot;, json.string(t.title)),\r\n  ])\r\n}\r\n\r\nfn decoder(\r\n  value: dynamic.Dynamic,\r\n) -&gt; Result(Todo, List(dynamic.DecodeError)) {\r\n  value\r\n  |&gt; dynamic.decode2(\r\n    Todo,\r\n    dynamic.field(&quot;id&quot;, dynamic.string),\r\n    dynamic.field(&quot;title&quot;, dynamic.string),\r\n  )\r\n}\r\n\r\npub fn main() {\r\n  let todo_converter = glitr.JsonConverter(encoder, decoder)\r\n  let get_all_todos_route = glitr.Route(\r\n    http.Get,\r\n    http.Http,\r\n    &quot;localhost&quot;,\r\n    2345,\r\n    False,\r\n    utils.simple_path_converter([&quot;todos&quot;]),\r\n    utils.no_body_converter(),\r\n    todo_converter\r\n  )\r\n\r\n  // This route can be shared and used by both frontend and backend applications\r\n\r\n  // Here is an example for a frontend application\r\n  let request = get_all_todos_route |&gt; utils.to_request(Nil, Nil)\r\n\r\n  // Use request as you wish (with glitr_lustre for example)\r\n}\r\n```\r\n\r\nFurther documentation can be found at &lt;https://hexdocs.pm/glitr&gt;.\r\n\r\n## Development\r\n\r\n```sh\r\ngleam run   # Run the project\r\ngleam test  # Run the tests\r\n```\r\n","url":"index.html"},{"doc":"glitr","title":"RouteOptions","content":"pub type RouteOptions(p, q, b) {\n  RouteOptions(path: p, query: q, body: b)\n}\n\nRouteOptions(path: p, query: q, body: b)\n\n\nSynonyms:\nglitr.RouteOptions\nglitr RouteOptions","url":"glitr.html#RouteOptions"},{"doc":"glitr","title":"glitr","content":"","url":"glitr.html"},{"doc":"glitr/error","title":"GlitrError","content":"pub type GlitrError {\n  RouteError(msg: String)\n  JsonDecodeError(err: json.DecodeError)\n}\n\nRouteError(msg: String)\n\n\nJsonDecodeError(err: json.DecodeError)\n\n\nSynonyms:\nglitr/error.GlitrError\nglitr/error GlitrError","url":"glitr/error.html#GlitrError"},{"doc":"glitr/error","title":"glitr/error","content":"","url":"glitr/error.html"},{"doc":"glitr/body","title":"BodyConverter","content":"pub type BodyConverter(body_type) {\n  BodyConverter(\n    encoder: fn(body_type) -&gt; string_builder.StringBuilder,\n    decoder: fn(String) -&gt; Result(body_type, error.GlitrError),\n  )\n}\n\nBodyConverter(\n  encoder: fn(body_type) -&gt; string_builder.StringBuilder,\n  decoder: fn(String) -&gt; Result(body_type, error.GlitrError),\n)\n\n\nSynonyms:\nglitr/body.BodyConverter\nglitr/body BodyConverter","url":"glitr/body.html#BodyConverter"},{"doc":"glitr/body","title":"BodyType","content":"pub type BodyType {\n  EmptyBody\n  StringBody\n  JsonBody\n}\n\nEmptyBody\n\n\nStringBody\n\n\nJsonBody\n\n\nSynonyms:\nglitr/body.BodyType\nglitr/body BodyType","url":"glitr/body.html#BodyType"},{"doc":"glitr/body","title":"RouteBody","content":"pub opaque type RouteBody(body_type)\n\n\nSynonyms:\nglitr/body.RouteBody\nglitr/body RouteBody","url":"glitr/body.html#RouteBody"},{"doc":"glitr/body","title":"decode","content":"pub fn decode(\n  body: RouteBody(a),\n  value: String,\n) -&gt; Result(a, GlitrError)\n\nSynonyms:\nglitr/body.decode\nglitr/body decode","url":"glitr/body.html#decode"},{"doc":"glitr/body","title":"empty_body","content":"pub fn empty_body() -&gt; RouteBody(Nil)\n\nSynonyms:\nglitr/body.empty_body\nglitr/body empty_body","url":"glitr/body.html#empty_body"},{"doc":"glitr/body","title":"encode","content":"pub fn encode(body: RouteBody(a), value: a) -&gt; StringBuilder\n\nSynonyms:\nglitr/body.encode\nglitr/body encode","url":"glitr/body.html#encode"},{"doc":"glitr/body","title":"get_type","content":"pub fn get_type(body: RouteBody(a)) -&gt; BodyType\n\nSynonyms:\nglitr/body.get_type\nglitr/body get_type","url":"glitr/body.html#get_type"},{"doc":"glitr/body","title":"json_body","content":"pub fn json_body(\n  encoder: fn(a) -&gt; Json,\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; RouteBody(a)\n\nSynonyms:\nglitr/body.json_body\nglitr/body json_body","url":"glitr/body.html#json_body"},{"doc":"glitr/body","title":"string_body","content":"pub fn string_body(converter: BodyConverter(a)) -&gt; RouteBody(a)\n\nSynonyms:\nglitr/body.string_body\nglitr/body string_body","url":"glitr/body.html#string_body"},{"doc":"glitr/body","title":"glitr/body","content":"","url":"glitr/body.html"},{"doc":"glitr/path","title":"PathConverter","content":"pub type PathConverter(path_type) {\n  PathConverter(\n    encoder: fn(path_type) -&gt; List(String),\n    decoder: fn(List(String)) -&gt; Result(path_type, Nil),\n  )\n}\n\nPathConverter(\n  encoder: fn(path_type) -&gt; List(String),\n  decoder: fn(List(String)) -&gt; Result(path_type, Nil),\n)\n\n\nSynonyms:\nglitr/path.PathConverter\nglitr/path PathConverter","url":"glitr/path.html#PathConverter"},{"doc":"glitr/path","title":"PathType","content":"pub type PathType {\n  StaticPath\n  ComplexPath\n}\n\nStaticPath\n\n\nComplexPath\n\n\nSynonyms:\nglitr/path.PathType\nglitr/path PathType","url":"glitr/path.html#PathType"},{"doc":"glitr/path","title":"RoutePath","content":"pub opaque type RoutePath(path_type)\n\n\nSynonyms:\nglitr/path.RoutePath\nglitr/path RoutePath","url":"glitr/path.html#RoutePath"},{"doc":"glitr/path","title":"complex_path","content":"pub fn complex_path(converter: PathConverter(a)) -&gt; RoutePath(a)\n\nSynonyms:\nglitr/path.complex_path\nglitr/path complex_path","url":"glitr/path.html#complex_path"},{"doc":"glitr/path","title":"decode","content":"pub fn decode(\n  path: RoutePath(a),\n  value: List(String),\n) -&gt; Result(a, Nil)\n\nSynonyms:\nglitr/path.decode\nglitr/path decode","url":"glitr/path.html#decode"},{"doc":"glitr/path","title":"encode","content":"pub fn encode(path: RoutePath(a), value: a) -&gt; List(String)\n\nSynonyms:\nglitr/path.encode\nglitr/path encode","url":"glitr/path.html#encode"},{"doc":"glitr/path","title":"get_type","content":"pub fn get_type(path: RoutePath(a)) -&gt; PathType\n\nSynonyms:\nglitr/path.get_type\nglitr/path get_type","url":"glitr/path.html#get_type"},{"doc":"glitr/path","title":"id_path","content":"pub fn id_path(root: List(String)) -&gt; RoutePath(String)\n\nSynonyms:\nglitr/path.id_path\nglitr/path id_path","url":"glitr/path.html#id_path"},{"doc":"glitr/path","title":"static_path","content":"pub fn static_path(root: List(String)) -&gt; RoutePath(Nil)\n\nSynonyms:\nglitr/path.static_path\nglitr/path static_path","url":"glitr/path.html#static_path"},{"doc":"glitr/path","title":"glitr/path","content":"","url":"glitr/path.html"},{"doc":"glitr/query","title":"QueryConverter","content":"pub type QueryConverter(query_type) {\n  QueryConverter(\n    encoder: fn(query_type) -&gt; List(#(String, String)),\n    decoder: fn(List(#(String, String))) -&gt;\n      Result(query_type, Nil),\n  )\n}\n\nQueryConverter(\n  encoder: fn(query_type) -&gt; List(#(String, String)),\n  decoder: fn(List(#(String, String))) -&gt; Result(query_type, Nil),\n)\n\n\nSynonyms:\nglitr/query.QueryConverter\nglitr/query QueryConverter","url":"glitr/query.html#QueryConverter"},{"doc":"glitr/query","title":"QueryType","content":"pub type QueryType {\n  EmptyQuery\n  ComplexQuery\n}\n\nEmptyQuery\n\n\nComplexQuery\n\n\nSynonyms:\nglitr/query.QueryType\nglitr/query QueryType","url":"glitr/query.html#QueryType"},{"doc":"glitr/query","title":"RouteQuery","content":"pub opaque type RouteQuery(query_type)\n\n\nSynonyms:\nglitr/query.RouteQuery\nglitr/query RouteQuery","url":"glitr/query.html#RouteQuery"},{"doc":"glitr/query","title":"complex_query","content":"pub fn complex_query(\n  converter: QueryConverter(a),\n) -&gt; RouteQuery(a)\n\nSynonyms:\nglitr/query.complex_query\nglitr/query complex_query","url":"glitr/query.html#complex_query"},{"doc":"glitr/query","title":"decode","content":"pub fn decode(\n  query: RouteQuery(a),\n  value: List(#(String, String)),\n) -&gt; Result(a, Nil)\n\nSynonyms:\nglitr/query.decode\nglitr/query decode","url":"glitr/query.html#decode"},{"doc":"glitr/query","title":"empty_query","content":"pub fn empty_query() -&gt; RouteQuery(Nil)\n\nSynonyms:\nglitr/query.empty_query\nglitr/query empty_query","url":"glitr/query.html#empty_query"},{"doc":"glitr/query","title":"encode","content":"pub fn encode(\n  query: RouteQuery(a),\n  value: a,\n) -&gt; List(#(String, String))\n\nSynonyms:\nglitr/query.encode\nglitr/query encode","url":"glitr/query.html#encode"},{"doc":"glitr/query","title":"get_type","content":"pub fn get_type(query: RouteQuery(a)) -&gt; QueryType\n\nSynonyms:\nglitr/query.get_type\nglitr/query get_type","url":"glitr/query.html#get_type"},{"doc":"glitr/query","title":"glitr/query","content":"","url":"glitr/query.html"},{"doc":"glitr/route","title":"Route","content":"pub type Route(\n  path_type,\n  query_type,\n  req_body_type,\n  res_body_type,\n) {\n  Route(\n    method: http.Method,\n    path: path.RoutePath(path_type),\n    query: query.RouteQuery(query_type),\n    req_body: body.RouteBody(req_body_type),\n    res_body: body.RouteBody(res_body_type),\n  )\n}\n\nRoute(\n  method: http.Method,\n  path: path.RoutePath(path_type),\n  query: query.RouteQuery(query_type),\n  req_body: body.RouteBody(req_body_type),\n  res_body: body.RouteBody(res_body_type),\n)\n\n\nSynonyms:\nglitr/route.Route\nglitr/route Route","url":"glitr/route.html#Route"},{"doc":"glitr/route","title":"new","content":"pub fn new() -&gt; Route(Nil, Nil, Nil, Nil)\n\nSynonyms:\nglitr/route.new\nglitr/route new","url":"glitr/route.html#new"},{"doc":"glitr/route","title":"with_method","content":"pub fn with_method(\n  route: Route(a, b, c, d),\n  method: Method,\n) -&gt; Route(a, b, c, d)\n\nSynonyms:\nglitr/route.with_method\nglitr/route with_method","url":"glitr/route.html#with_method"},{"doc":"glitr/route","title":"with_path","content":"pub fn with_path(\n  route: Route(a, b, c, d),\n  path: RoutePath(e),\n) -&gt; Route(e, b, c, d)\n\nSynonyms:\nglitr/route.with_path\nglitr/route with_path","url":"glitr/route.html#with_path"},{"doc":"glitr/route","title":"with_query","content":"pub fn with_query(\n  route: Route(a, b, c, d),\n  query: RouteQuery(e),\n) -&gt; Route(a, e, c, d)\n\nSynonyms:\nglitr/route.with_query\nglitr/route with_query","url":"glitr/route.html#with_query"},{"doc":"glitr/route","title":"with_request_body","content":"pub fn with_request_body(\n  route: Route(a, b, c, d),\n  req_body: RouteBody(e),\n) -&gt; Route(a, b, e, d)\n\nSynonyms:\nglitr/route.with_request_body\nglitr/route with_request_body","url":"glitr/route.html#with_request_body"},{"doc":"glitr/route","title":"with_response_body","content":"pub fn with_response_body(\n  route: Route(a, b, c, d),\n  res_body: RouteBody(e),\n) -&gt; Route(a, b, c, e)\n\nSynonyms:\nglitr/route.with_response_body\nglitr/route with_response_body","url":"glitr/route.html#with_response_body"},{"doc":"glitr/route","title":"glitr/route","content":"","url":"glitr/route.html"},{"doc":"glitr/service","title":"RouteService","content":"pub type RouteService(base_type, upsert_type) {\n  RouteService(\n    root_path: List(String),\n    base: Option(\n      #(\n        fn(base_type) -&gt; json.Json,\n        fn(dynamic.Dynamic) -&gt;\n          Result(base_type, List(dynamic.DecodeError)),\n      ),\n    ),\n    upsert: Option(\n      #(\n        fn(upsert_type) -&gt; json.Json,\n        fn(dynamic.Dynamic) -&gt;\n          Result(upsert_type, List(dynamic.DecodeError)),\n      ),\n    ),\n  )\n}\n\nRouteService(\n  root_path: List(String),\n  base: Option(\n    #(\n      fn(base_type) -&gt; json.Json,\n      fn(dynamic.Dynamic) -&gt;\n        Result(base_type, List(dynamic.DecodeError)),\n    ),\n  ),\n  upsert: Option(\n    #(\n      fn(upsert_type) -&gt; json.Json,\n      fn(dynamic.Dynamic) -&gt;\n        Result(upsert_type, List(dynamic.DecodeError)),\n    ),\n  ),\n)\n\n\nSynonyms:\nglitr/service.RouteService\nglitr/service RouteService","url":"glitr/service.html#RouteService"},{"doc":"glitr/service","title":"create_route","content":"pub fn create_route(\n  service: RouteService(a, b),\n) -&gt; Result(Route(Nil, Nil, b, a), GlitrError)\n\nSynonyms:\nglitr/service.create_route\nglitr/service create_route","url":"glitr/service.html#create_route"},{"doc":"glitr/service","title":"delete_route","content":"pub fn delete_route(\n  service: RouteService(a, b),\n) -&gt; Result(Route(String, Nil, Nil, String), GlitrError)\n\nSynonyms:\nglitr/service.delete_route\nglitr/service delete_route","url":"glitr/service.html#delete_route"},{"doc":"glitr/service","title":"get_all_route","content":"pub fn get_all_route(\n  service: RouteService(a, b),\n) -&gt; Result(Route(Nil, Nil, Nil, List(a)), GlitrError)\n\nSynonyms:\nglitr/service.get_all_route\nglitr/service get_all_route","url":"glitr/service.html#get_all_route"},{"doc":"glitr/service","title":"get_route","content":"pub fn get_route(\n  service: RouteService(a, b),\n) -&gt; Result(Route(String, Nil, Nil, a), GlitrError)\n\nSynonyms:\nglitr/service.get_route\nglitr/service get_route","url":"glitr/service.html#get_route"},{"doc":"glitr/service","title":"new","content":"pub fn new() -&gt; RouteService(a, b)\n\nSynonyms:\nglitr/service.new\nglitr/service new","url":"glitr/service.html#new"},{"doc":"glitr/service","title":"update_route","content":"pub fn update_route(\n  service: RouteService(a, b),\n) -&gt; Result(Route(String, Nil, b, a), GlitrError)\n\nSynonyms:\nglitr/service.update_route\nglitr/service update_route","url":"glitr/service.html#update_route"},{"doc":"glitr/service","title":"with_base_type","content":"pub fn with_base_type(\n  service: RouteService(a, b),\n  base_encoder: fn(c) -&gt; Json,\n  base_decoder: fn(Dynamic) -&gt; Result(c, List(DecodeError)),\n) -&gt; RouteService(c, b)\n\nSynonyms:\nglitr/service.with_base_type\nglitr/service with_base_type","url":"glitr/service.html#with_base_type"},{"doc":"glitr/service","title":"with_root_path","content":"pub fn with_root_path(\n  service: RouteService(a, b),\n  root_path: List(String),\n) -&gt; RouteService(a, b)\n\nSynonyms:\nglitr/service.with_root_path\nglitr/service with_root_path","url":"glitr/service.html#with_root_path"},{"doc":"glitr/service","title":"with_upsert_type","content":"pub fn with_upsert_type(\n  service: RouteService(a, b),\n  upsert_encoder: fn(c) -&gt; Json,\n  upsert_decoder: fn(Dynamic) -&gt; Result(c, List(DecodeError)),\n) -&gt; RouteService(a, c)\n\nSynonyms:\nglitr/service.with_upsert_type\nglitr/service with_upsert_type","url":"glitr/service.html#with_upsert_type"},{"doc":"glitr/service","title":"glitr/service","content":"","url":"glitr/service.html"},{"doc":"glitr/utils","title":"to_request","content":"pub fn to_request(\n  route: Route(a, b, c, d),\n  path: a,\n  query: b,\n  body: c,\n) -&gt; Request(String)\n\nSynonyms:\nglitr/utils.to_request\nglitr/utils to_request","url":"glitr/utils.html#to_request"},{"doc":"glitr/utils","title":"glitr/utils","content":"","url":"glitr/utils.html"}]);